@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Login Screen
 #
-# $Id: lonlogin.pm,v 1.85 2006/12/05 02:55:56 albertel Exp $
+# $Id: lonlogin.pm,v 1.87 2007/01/14 02:00:04 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -40,17 +40,6 @@ use Apache::migrateuser();
 use lib '/home/httpd/lib/perl/';
 use LONCAPA;
  
-
-sub additional_machine_domains {
-    my @domains;
-    open(my $fh,"<".$Apache::lonnet::perlvar{'lonTabDir'}.'/expected_domains.tab');
-    while( my $line = <$fh>) {
-	$line =~ s/\s//g;
-	push(@domains,$line);
-    }
-    return @domains;
-}
-
 sub handler {
     my $r = shift;
 
@@ -137,12 +126,7 @@ ENDFAILED
     }
     my $iconpath= 'http://'.$ENV{'HTTP_HOST'}.':'.$port_to_use.
                   $r->dir_config('lonIconsURL');
-    my $domain  = $r->dir_config('lonDefDomain');
-    my $testdomain=(split(/\./,$ENV{'HTTP_HOST'}))[0];
-    foreach my $posdom (&Apache::lonnet::current_machine_domains(),
-			&additional_machine_domains()) {
-	if (lc($posdom) eq lc($testdomain)) { $domain=$posdom; }
-    }
+    my $domain = &Apache::lonnet::default_login_domain();
     if (($env{'form.domain'}) && 
 	 ($Apache::lonnet::domaindescription{$env{'form.domain'}})) {
 	$domain=$env{'form.domain'};
@@ -419,24 +403,25 @@ ENDSERVERFORM
    
    <!-- A cell that will hold the 'access', 'about', and 'catalog' links -->
    <!-- Row 3 Column 1 -->
-   <td valign="top" height=60 align="left" bgcolor="$sidebg">
+   <td valign="top" height="60" align="left" bgcolor="$sidebg">
     <table cellpadding="0" cellspacing="2" border="0">
      <tr>
       <td>&nbsp;</td>
       <td><a href="/adm/login?interface=textual"><b>$lt{'access'}</b></a></td>
-    </tr>
-    <tr>
-     <td>&nbsp;</td>
-     <td><a href="/adm/about.html"><b>$lt{'about'}</b></a></td>
-    </tr>
-    <tr>
-     <td>&nbsp;</td>
-     <td><a href="/adm/coursecatalog"><b>$lt{'catalog'}</b></a></td>
-    </tr>
-    <tr>
+     </tr>
+     <tr>
+      <td>&nbsp;</td>
+      <td><a href="/adm/about.html"><b>$lt{'about'}</b></a></td>
+     </tr>
+     <tr>
+      <td>&nbsp;</td>
+      <td><a href="/adm/coursecatalog"><b>$lt{'catalog'}</b></a></td>
+     </tr>
+     <tr>
       <td colspan="2">&nbsp;</td>
-    </tr>
-   </table>
+     </tr>
+    </table>
+   </td>
    <!-- The shaded space between the two main columns -->
    <!-- Row 3 Column 2 -->
    <td width=27 height=60 background="$iconpath/fillleft.gif"><img src="$iconpath/fillleft.gif" alt="" /></td>
