@@ -1,3 +1,7 @@
+# The LearningOnline Network with CAPA
+# Handler for displaying the course catalog interface
+#
+# $Id: coursecatalog.pm,v 1.19 2007/01/14 01:59:37 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -55,7 +59,14 @@ sub handler {
     &Apache::lonacc::get_posted_cgi($r);
     &Apache::lonlocal::get_language_handle($r);
     &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['sortby']);
-    my $codedom = $Apache::lonnet::perlvar{'lonDefDomain'};
+    my $codedom = &Apache::lonnet::default_login_domain(); 
+
+    if (($env{'user.domain'} ne '') && ($env{'user.domain'} ne 'public')) { 
+        $codedom = $env{'user.domain'};
+        if ($env{'request.role.domain'} ne '') {
+            $codedom = $env{'request.role.domain'};
+        }
+    }
     my $formname = 'coursecatalog';
     my $domdesc = $Apache::lonnet::domaindescription{$codedom};
 
@@ -63,9 +74,9 @@ sub handler {
     if ($env{'form.coursenum'} ne '' && &user_is_known()) {
         &course_details($r,$codedom,$formname,$domdesc);
     } else {
-        &course_selector($r,$codedom,$formname,$domdesc);
+        my $numtitles = &course_selector($r,$codedom,$formname,$domdesc);
         if ($env{'form.state'} eq 'listing') {
-            $r->print('<br /><br />'.&print_course_listing($codedom).'<br />');
+            $r->print(&print_course_listing($codedom,$numtitles).'<br />');
         }
     }
     $r->print(&Apache::loncommon::end_page());
@@ -83,10 +94,13 @@ sub course_details {
                                              'add_entries' => \%add_entries,
                                              'no_inline_link'   => 1,});
     $r->print($start_page);
+    if ($env{'form.numtitles'} > 0) {
+        &Apache::lonhtmlcommon::add_breadcrumb
+                ({href=>"/adm/coursecatalog",
+                  text=>"Select courses"});
+    }
     &Apache::lonhtmlcommon::add_breadcrumb
-            ({href=>"/adm/coursecatalog",
-              text=>"Select courses"},
-             {href=>"javascript:document.$formname.submit()",
+             ({href=>"javascript:document.$formname.submit()",
               text=>"Course listing"},
              {text=>"Course details"});
     $r->print(&Apache::lonhtmlcommon::breadcrumbs('Course Details'));
@@ -133,41 +147,49 @@ sub course_selector {
             if ($numtitles > 4) {
                 $lasttitle = 4;
             }
-            my @data = ('top');
-            for (my $k=0; $k<$lasttitle; $k++) {
-                my $cat = $codetitles[$k];
-                my $level = 1;
-                $level = &recurse_options($codetitles[$k],$idlist{$codetitles[$k]},$level,$cat,\%cat_items,\@data,\%by_year,\%by_sem,\%by_dept);     }
-            $scripttext .= &build_javascript(\%by_year,\%by_sem,\%by_dept,\%cat_order,\@codetitles);
-            $jscript .= &javascript_select_filler($formname,$scripttext,\@codetitles,$longtitles_str,$allidlist);
-        }
-        if ($env{'form.state'} eq 'listing') {
-            $jscript .= '
+            if ($numtitles == 0) {
+                if (!defined($env{'form.state'})) {
+                    $env{'form.state'} = 'listing';
+                }
+            } else {
+                my @data = ('top');
+                for (my $k=0; $k<$lasttitle; $k++) {
+                    my $cat = $codetitles[$k];
+                    my $level = 1;
+                    $level = &recurse_options($codetitles[$k],$idlist{$codetitles[$k]},$level,$cat,\%cat_items,\@data,\%by_year,\%by_sem,\%by_dept);
+                }
+                $scripttext .= &build_javascript(\%by_year,\%by_sem,\%by_dept,\%cat_order,\@codetitles);
+                $jscript .= &javascript_select_filler($formname,$scripttext,\@codetitles,$longtitles_str,$allidlist);
+                if ($env{'form.state'} eq 'listing') {
+                    $jscript .= '
 function setElements() {
 ';
-            for (my $i=0; $i<@codetitles-1; $i++) {
-                if ($env{'form.'.$codetitles[$i]} != -1) {
-                    $jscript .= '
+                    for (my $i=0; $i<@codetitles-1; $i++) {
+                        if ($env{'form.'.$codetitles[$i]} != -1) {
+                            $jscript .= '
     for (var j=0; j<document.'.$formname.'.'.$codetitles[$i].'.length; j++) {
         if (document.'.$formname.'.'.$codetitles[$i].'[j].value == "'.$env{'form.'.$codetitles[$i]}.'") {
             document.'.$formname.'.'.$codetitles[$i].'.selectedIndex = j;
         }
     }
 ';
-                }
-            }
-            $jscript .= '   courseSet()'."\n";
-            if ($env{'form.'.$codetitles[-1]} != -1) {
-                $jscript .= '
+                        }
+                    }
+                    $jscript .= '   courseSet()'."\n";
+                    if ($env{'form.'.$codetitles[-1]} != -1) {
+                        $jscript .= '
     for (var j=0; j<document.'.$formname.'.'.$codetitles[-1].'.length; j++) {
         if (document.'.$formname.'.'.$codetitles[-1].'[j].value == "'.$env{'form.'.$codetitles[-1]}.'") {
             document.'.$formname.'.'.$codetitles[-1].'.selectedIndex = j;
         }
     }
 ';
+                    }
+                    $jscript .= '}';
+                }
             }
-            $jscript .= '}';
-            $jscript .= qq|
+            if ($env{'form.state'} eq 'listing') {
+                $jscript .= qq|
 function changeSort(caller) {
     document.$formname.sortby.value = caller;
     document.$formname.submit();
@@ -176,12 +198,13 @@ function setCourseId(caller) {
    document.$formname.coursenum.value = caller;
    document.$formname.submit(); 
 }\n|;
+            }
         }
         my $js = '<script type"text/javascript">'."\n$jscript\n".
                  '</script>';
         my %add_entries = (topmargin    => "0",
                            marginheight => "0",);
-        if ($env{'form.state'} eq 'listing') {
+        if (($env{'form.state'} eq 'listing') && ($numtitles > 0)) {
             $add_entries{'onLoad'} = 'setElements()';
         }
         my $start_page =
@@ -191,11 +214,16 @@ function setCourseId(caller) {
                                              'no_inline_link'   => 1,});
         $r->print($start_page);
         if ($env{'form.state'} eq 'listing') {
-            &Apache::lonhtmlcommon::add_breadcrumb
-            ({href=>"/adm/coursecatalog",
-              text=>"Select courses"},
-             {text=>"Course listing"});
-             $r->print(&Apache::lonhtmlcommon::breadcrumbs('Course Listing'));
+            if ($numtitles > 0) {
+                &Apache::lonhtmlcommon::add_breadcrumb
+                ({href=>"/adm/coursecatalog",
+                  text=>"Select courses"},
+                 {text=>"Course listing"});
+            } else {
+                &Apache::lonhtmlcommon::add_breadcrumb
+                ({text=>"Course listing"});
+            }
+            $r->print(&Apache::lonhtmlcommon::breadcrumbs('Course Listing'));
         } else {
             &Apache::lonhtmlcommon::add_breadcrumb
             ({href=>"/adm/coursecatalog",
@@ -203,9 +231,9 @@ function setCourseId(caller) {
             $r->print(&Apache::lonhtmlcommon::breadcrumbs('Select courses'));
         }
         $r->print('<h3>'.&mt('Display information about official [_1] classes for which LON-CAPA courses have been created:',$domdesc).'</h3>');
-        $r->print(&mt('<b>Choose which course(s) to list.</b><br />'));
         $r->print('<form name="coursecatalog" method="post">');
         if ($numtitles > 0) {
+            $r->print(&mt('<b>Choose which course(s) to list.</b><br />'));
             $r->print('<table><tr>');
             for (my $k=0; $k<$lasttitle-1; $k++) {
                 my @unsorted = @{$cat_items{$codetitles[$k]}};
@@ -236,22 +264,50 @@ function setCourseId(caller) {
                       '</option>'."\n".'</select>'."\n".
                  '</td>'
                 );
+            if (&user_is_dc($codedom)) {
+                my $showdetails_status;
+                if ($env{'form.showdetails'}) {
+                    $showdetails_status = 'checked="checked" ';
+                } 
+                $r->print('<td></td><td><input type="checkbox" name="showdetails" value="1" '.$showdetails_status.'/>'.&mt('Show full details for each course (DC only)').'</td>');
+            }
             $r->print('</tr></table>');
             if ($numtitles > 4) {
                 $r->print('<br /><br />'.$codetitles[$numtitles-1].'<br />'."\n".
                 '<input type="text" name="'.$codetitles[$numtitles-1].'" /><br />'."\n");
             }
+            $r->print('<br />');
+        }
+        $r->print('<input type="hidden" name="coursenum" value="" /><input type="hidden" name="sortby" value="" /><input type="hidden" name="state" value="listing" />');
+        if ($numtitles > 0) {
+            $r->print('<input type="submit" name="catalogfilter" value="'.&mt('Display courses').'" />');
+        }
+        $r->print('<input type="hidden" name="numtitles" value="'.
+                  $numtitles.'" /></form>');
+        if (($numtitles > 0) && ($env{'form.state'} eq 'listing')) {
+            $r->print('<br /><br />');
         }
-        $r->print('<br /><input type="hidden" name="coursenum" value="" /><input type="hidden" name="state" value="listing" /><input type="hidden" name="sortby" value="" /><input type="submit" name="catalogfilter" value="'.&mt('Display courses').'" /></form>');
     } else {
         $r->print(&Apache::loncommon::start_page('Course Catalog','',
                   {
                    'no_inline_link'   => 1,}));
         $r->print('<br />'.&mt('No official courses to display for [_1].',$domdesc));
     }
-    return;
+    return $numtitles;
 }
 
+sub user_is_dc {
+    my ($codedom) = @_;
+    if (exists($env{'user.role.dc./'.$codedom.'/'})) {
+        my $livedc = 1;
+        my $now = time;
+        my ($start,$end)=split(/\./,$env{'user.role.dc./'.$codedom.'/'});
+        if ($start && $start>$now) { $livedc = 0; }
+        if ($end   && $end  <$now) { $livedc = 0; }
+        return $livedc;
+    }
+    return;
+}
 
 sub recurse_options {
     my ($currkey,$currlist,$level,$cat,$cat_options,$data,$by_year,$by_sem,$by_dept) = @_;
@@ -337,23 +393,28 @@ sub build_javascript {
 }
 
 sub search_courselist {
-    my ($domain) = @_;
-    my ($instcode,%codedefaults,@code_order);
-    my $defaults_result = 
-        &Apache::lonnet::auto_instcode_defaults($domain,\%codedefaults,
-                                               \@code_order);
-    if ($defaults_result eq 'ok') {
-        $instcode ='^';
-        foreach my $item (@code_order) {
-            if ($env{'form.'.$item} eq '0' ) {
-                $instcode .= $codedefaults{$item}; 
-            } else {
-                $instcode .= $env{'form.'.$item};
-            }
-        }
-        $instcode .= '$';
+    my ($domain,$numtitles) = @_;
+    my $instcode;
+    if (defined($numtitles) && $numtitles == 0) {
+        $instcode = '.+';
     } else {
-        $instcode = '.';
+        my (%codedefaults,@code_order);
+        my $defaults_result = 
+            &Apache::lonnet::auto_instcode_defaults($domain,\%codedefaults,
+                                                    \@code_order);
+        if ($defaults_result eq 'ok') {
+            $instcode ='^';
+            foreach my $item (@code_order) {
+                if ($env{'form.'.$item} eq '0' ) {
+                    $instcode .= $codedefaults{$item}; 
+                } else {
+                    $instcode .= $env{'form.'.$item};
+                }
+            }
+            $instcode .= '$';
+        } else {
+            $instcode = '.';
+        }
     }
     my %courses = &Apache::lonnet::courseiddump($domain,'.',1,$instcode,'.','.',
                                                 undef,undef,'Course',1);
@@ -362,9 +423,16 @@ sub search_courselist {
 
 
 sub print_course_listing {
-    my ($domain) = @_;
+    my ($domain,$numtitles) = @_;
     my $output;
     my %courses;
+    my $knownuser = &user_is_known();
+    my $details = $env{'form.coursenum'};
+    if (&user_is_dc($domain)) {
+        if ($env{'form.showdetails'}) {
+            $details = 1;
+        }
+    }
     if ($env{'form.coursenum'} ne '') {
         %courses = &Apache::lonnet::courseiddump($domain,'.',1,'.','.',
                                                  $env{'form.coursenum'},
@@ -374,24 +442,34 @@ sub print_course_listing {
             return $output;
         }
     } else {
-        %courses = &search_courselist($domain);
+        %courses = &search_courselist($domain,$numtitles);
         if (keys(%courses) == 0) {
             $output = &mt('No courses match the criteria you selected.');
             return $output;
         }
-        if (&user_is_known()) {
+        if ($knownuser && !$env{'form.showdetails'}) {
             $output = &mt('<b>Note for students:</b> If you are officially enrolled in a course but the course is not listed in your LON-CAPA courses, click the "Show more details" link for the specific course and check the default access dates and/or automated enrollment settings.<br /><br />');
         }
     }
-    $output .= &Apache::loncommon::start_data_table().
-               &Apache::loncommon::start_data_table_header_row();
-    my @coltitles = ('Code','Sections','Crosslisted','Title','Owner');
+    $output .= &construct_data_table($knownuser,\%courses,$details);
+    $output .= &Apache::lonhtmlcommon::echo_form_input(['coursenum','state','catalogfilter','sortby','showdetails']);
+    return $output;
+}
+
+sub construct_data_table {
+    my ($knownuser,$courses,$details,$usersections) = @_;
     my %sortname;
-    if ($env{'form.coursenum'} eq '') {
+    if (($details eq '') || ($env{'form.showdetails'})) {
         $sortname{'Code'} = 'code';
         $sortname{'Title'} = 'title';
         $sortname{'Owner'} = 'owner';
     }
+    my $output = &Apache::loncommon::start_data_table().
+                 &Apache::loncommon::start_data_table_header_row();
+    my @coltitles = ('Code','Sections','Crosslisted','Title','Owner');
+    if (ref($usersections) eq 'HASH') {
+       $coltitles[1] = 'Your Section';
+    }
     foreach my $item (@coltitles) {
         $output .= '<th>';
         if (defined($sortname{$item})) {
@@ -401,20 +479,21 @@ sub print_course_listing {
         }
         $output .= '</th>';
     }
-    if (&user_is_known()) {
-        if ($env{'form.coursenum'} eq '') {
-            $output .= '<th>&nbsp;</th>';
-        } else {
+    if ($knownuser) {
+        if ($details) {
             $output .=
               '<th>'.&mt('Default Access Dates for Students').'</th>'.
               '<th>'.&mt('Student Counts').'</th>'.
               '<th>'.&mt('Auto-enrollment of <br />registered students').'</th>';
+        } else {
+            $output .= '<th>&nbsp;</th>';
         }
     }
     &Apache::loncommon::end_data_table_header_row();
-    my %courseinfo = &build_courseinfo_hash(%courses);
+    my %courseinfo = &build_courseinfo_hash($courses,$knownuser,$details,
+                                            $usersections);
     my %Sortby;
-    foreach my $course (sort(keys(%courses))) {
+    foreach my $course (sort(keys(%{$courses}))) {
         if ($env{'form.sortby'} eq 'code') {
             push(@{$Sortby{$courseinfo{$course}{'code'}}},$course);
         } elsif ($env{'form.sortby'} eq 'owner') {
@@ -432,33 +511,31 @@ sub print_course_listing {
     foreach my $item (@sorted_courses) {
         foreach my $course (@{$Sortby{$item}}) {
             $output.=&Apache::loncommon::start_data_table_row(); 
-            $output.=&courseinfo_row($courseinfo{$course});
+            $output.=&courseinfo_row($courseinfo{$course},$knownuser,$details);
             $output.=&Apache::loncommon::end_data_table_row();
         }
     }
     $output .= &Apache::loncommon::end_data_table();
-    my $echo = &Apache::lonhtmlcommon::echo_form_input(['coursenum','state','catalogfilter','sortby']);
-    $output .= $echo;
     return $output;
 }
 
 sub build_courseinfo_hash {
-    my (%courses) = @_;
+    my ($courses,$knownuser,$details,$usersections) = @_;
     my %courseinfo;
     my $now = time;
-    foreach my $course (keys(%courses)) {
+    foreach my $course (keys(%{$courses})) {
         my $descr;
-        if ($courses{$course} =~ m/^([^:]*):/i) {
+        if ($courses->{$course} =~ m/^([^:]*):/i) {
             $descr = &unescape($1);
         } else {
-            $descr = &unescape($courses{$course});
+            $descr = &unescape($courses->{$course});
         }
         my $cleandesc=&HTML::Entities::encode($descr,'<>&"');
         $cleandesc=~s/'/\\'/g;
         $cleandesc =~ s/^\s+//;
         my ($cdom,$cnum)=split(/\_/,$course);
 
-        my ($desc,$instcode,$owner,$ttype) = split(/:/,$courses{$course});
+        my ($desc,$instcode,$owner,$ttype) = split(/:/,$courses->{$course});
         $owner = &unescape($owner);
         my ($ownername,$ownerdom);
         if ($owner =~ /:/) {
@@ -483,41 +560,47 @@ sub build_courseinfo_hash {
         my %coursehash = &Apache::lonnet::dump('environment',$cdom,$cnum);
         my @classids;
         my @crosslistings;
-        my $seclist = &identify_sections($coursehash{'internal.sectionnums'});
+        my ($seclist,$numsec) = 
+            &identify_sections($coursehash{'internal.sectionnums'});
+        if (ref($usersections) eq 'HASH') {
+            if (ref($usersections->{$course}) eq 'ARRAY') {
+                $seclist = join(', ',@{$usersections->{$course}});
+            }
+        }
         $courseinfo{$course}{'seclist'} = $seclist;
-        my $xlist_items = &identify_sections($coursehash{'internal.crosslistings'});
+        my ($xlist_items,$numxlist) = 
+            &identify_sections($coursehash{'internal.crosslistings'});
         my $showsyllabus = 1; # default is to include a syllabus link
         if (defined($coursehash{'showsyllabus'})) {
             $showsyllabus = $coursehash{'showsyllabus'};
         }
         $courseinfo{$course}{'showsyllabus'} = $showsyllabus;
-        if (defined($env{'form.coursenum'})) {
-            if ($cnum eq $env{'form.coursenum'}) {
-                $courseinfo{$course}{'counts'} =  &count_students($cdom,$cnum);
-                $courseinfo{$course}{'autoenrollment'} =
-                   &autoenroll_info(\%coursehash,$now,$seclist,$xlist_items,
-                                    $instcode,$owner,$cdom,$cnum);
+        if (((defined($env{'form.coursenum'}) && ($cnum eq $env{'form.coursenum'}))) ||
+            ($knownuser && ($details == 1))) {  
+            $courseinfo{$course}{'counts'} =  &count_students($cdom,$cnum,$numsec);
+            $courseinfo{$course}{'autoenrollment'} =
+                &autoenroll_info(\%coursehash,$now,$seclist,$xlist_items,
+                                 $instcode,$owner,$cdom,$cnum);
 
-               my $startaccess = '';
-               my $endaccess = '';
-               my $accessdates;
-               if ( defined($coursehash{'default_enrollment_start_date'}) ) {
-                   $startaccess = &Apache::lonlocal::locallocaltime($coursehash{'default_enrollment_start_date'});
-               }
-               if ( defined($coursehash{'default_enrollment_end_date'}) ) {
-                   $endaccess = &Apache::lonlocal::locallocaltime($coursehash{'default_enrollment_end_date'});
-                   if ($coursehash{'default_enrollment_end_date'} == 0) {
-                       $endaccess = "No ending date";
-                   }
-               }
-               if ($startaccess) {
-                   $accessdates .= &mt('<i>From:</i> ').$startaccess.'<br />';
-               }
-               if ($endaccess) {
-                   $accessdates .= &mt('<i>To:</i> ').$endaccess.'<br />';
-               }
-               $courseinfo{$course}{'access'} = $accessdates;
+            my $startaccess = '';
+            my $endaccess = '';
+            my $accessdates;
+            if ( defined($coursehash{'default_enrollment_start_date'}) ) {
+                $startaccess = &Apache::lonlocal::locallocaltime($coursehash{'default_enrollment_start_date'});
             }
+            if ( defined($coursehash{'default_enrollment_end_date'}) ) {
+                $endaccess = &Apache::lonlocal::locallocaltime($coursehash{'default_enrollment_end_date'});
+                if ($coursehash{'default_enrollment_end_date'} == 0) {
+                    $endaccess = "No ending date";
+                }
+            }
+            if ($startaccess) {
+                $accessdates .= &mt('<i>From:</i> ').$startaccess.'<br />';
+            }
+            if ($endaccess) {
+                $accessdates .= &mt('<i>To:</i> ').$endaccess.'<br />';
+            }
+            $courseinfo{$course}{'access'} = $accessdates;
         }
         if ($xlist_items eq '') {
             $xlist_items = &mt('No');
@@ -528,7 +611,7 @@ sub build_courseinfo_hash {
 }
 
 sub count_students {
-    my ($cdom,$cnum) = @_;
+    my ($cdom,$cnum,$numsec) = @_;
     my $classlist = &Apache::loncoursedata::get_classlist($cdom,$cnum);
     my %student_count = (
                            Active => 0,
@@ -547,7 +630,7 @@ sub count_students {
         $student_count{$data->[$idx{'status'}]} ++;
     }
 
-    my $countslist;
+    my $countslist = &mt('[quant,_1,section]',$numsec).':<br />';
     foreach my $status ('Active','Future') {
         $countslist .= '<nobr>'.$status_title{$status}.': '.
                        $student_count{$status}.'</nobr><br />';
@@ -556,7 +639,7 @@ sub count_students {
 }
 
 sub courseinfo_row {
-    my ($info) = @_;
+    my ($info,$knownuser,$details) = @_;
     my ($cdom,$cnum,$title,$ownerlast,$code,$owner,$seclist,$xlist_items,
         $accessdates,$showsyllabus,$counts,$autoenrollment,$output);
     if (ref($info) eq 'HASH') {
@@ -588,14 +671,14 @@ sub courseinfo_row {
     }
     $output .= '</font></td>'.
                '<td>'.$ownerlast.'</td>';
-    if (&user_is_known()) {
-        if ($env{'form.coursenum'} eq '') {
-            $output .= "<td><a href=\"javascript:setCourseId('$cnum')\">".&mt('Show more details').'</a></td>';
-        } else { 
+    if ($knownuser) {
+        if ($details) {
             $output .=
                '<td>'.$accessdates.'</td>'. 
                '<td>'.$counts.'</td>'.
                '<td>'.$autoenrollment.'</td>';
+        } else {
+            $output .= "<td><a href=\"javascript:setCourseId('$cnum')\">".&mt('Show more details').'</a></td>';
         }
     }
     return $output;
@@ -620,7 +703,8 @@ sub identify_sections {
     }
     @secnums = sort {$a <=> $b} @secnums;
     my $seclist = join(', ',@secnums);
-    return $seclist;
+    my $numsec = @secnums;
+    return ($seclist,$numsec);
 }
 
 sub get_valid_classes {
@@ -656,11 +740,11 @@ sub get_valid_classes {
     if ($totalitems > 0) {
         if (@{$validations{'sections'}}) {
             $response = &mt('Sections: ').
-                        join(',',@{$validations{'sections'}}).'<br />';
+                        join(', ',@{$validations{'sections'}}).'<br />';
         }
         if (@{$validations{'xlists'}}) {
             $response .= &mt('Courses: ').
-                        join(',',@{$validations{'xlists'}});
+                        join(', ',@{$validations{'xlists'}});
         }
     }
     return $response;
