@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # handler for DC-only modifiable course settings
 #
-# $Id: lonmodifycourse.pm,v 1.28 2006/11/14 20:38:21 raeburn Exp $
+# $Id: lonmodifycourse.pm,v 1.30 2007/01/15 01:08:53 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -67,7 +67,7 @@ sub get_enrollment_settings {
                 } else {
                     $enrollvar{$type} = localtime($settings{$item});
                 }
-            } elsif ($type eq "sectionums") {
+            } elsif ($type eq "sectionnums") {
                 $enrollvar{$type} = $settings{$item};
                 $enrollvar{$type} =~ s/,/, /g;
             } elsif ($type eq "authtype"
@@ -138,6 +138,9 @@ sub print_course_selection_page {
                       'ownerdomfilter','coursefilter'];
     my %filter;
     my $type = $env{'form.type'};
+    if ($type eq '') {
+        $type = 'Course';
+    }
     my $action = '/adm/modifycourse';
     my $dctitle = &Apache::lonnet::plaintext('dc');
     $r->print(&mt('Revise your search criteria for this domain').' ('.$domdesc.').<br /><br />');
@@ -421,6 +424,9 @@ all settings except course code, course owner, and default authentication method
     }
     &print_header($r,$cdesc,$javascript_validations);
     my $type = $env{'form.type'};
+    if ($type eq '') {
+        $type = 'Course';
+    }
     my $dctitle = &Apache::lonnet::plaintext('dc');
     my $cctitle = &Apache::lonnet::plaintext('cc',$type);
     my $mainheader = &mt('Course settings modifiable by [_1] only.',$dctitle);
@@ -477,7 +483,7 @@ ENDDOCUMENT
 }
 
 sub modify_course {
-    my ($r,$cdom,$cnum,$cdesc,$domdesc) = @_;
+    my ($r,$cdom,$cnum,$cdesc,$domdesc,$type) = @_;
     my %longtype = &course_settings_descrip();
     my %settings = &Apache::lonnet::get('environment',['internal.courseowner','internal.coursecode','internal.authtype','internal.autharg','internal.sectionnums','internal.crosslistings','description'],$cdom,$cnum);
     my %currattr = ();
@@ -554,7 +560,7 @@ sub modify_course {
         }
     }
     if ($changeowner == 1 || $changecode == 1) {
-        my $courseid_entry = &escape($cdom.'_'.$cnum).'='.&escape($description).':'.&escape($env{'form.coursecode'}).':'.&escape($env{'form.courseowner'});
+        my $courseid_entry = &escape($cdom.'_'.$cnum).'='.&escape($description).':'.&escape($env{'form.coursecode'}).':'.&escape($env{'form.courseowner'}).':'.&escape($type);
         &Apache::lonnet::courseidput($cdom,$courseid_entry,&Apache::lonnet::homeserver($cnum,$cdom));
     }
 
@@ -878,6 +884,9 @@ sub handler {
                   text=>"Pick action"});
                 my ($checked,$cdesc) = &check_course($r,$dom,$domdesc);
                 my $type = $env{'form.type'};
+                if ($type eq '') {
+                    $type = 'Course';
+                }
                 if ($checked eq 'ok') {
                     if ($phase eq 'menu') {
                         &print_modification_menu($r,$cdesc);
@@ -913,7 +922,7 @@ sub handler {
                             &Apache::lonhtmlcommon::add_breadcrumb
                             ({href=>"javascript:changePage(document.$phase,'$phase')",
                               text=>"Result"});
-                            &modify_course($r,$cdom,$cnum,$cdesc,$domdesc);
+                            &modify_course($r,$cdom,$cnum,$cdesc,$domdesc,$type);
                         }
                     }
                 } else {
