@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Publication Handler
 #
-# $Id: lonpublisher.pm,v 1.215 2006/12/06 22:22:39 albertel Exp $
+# $Id: lonpublisher.pm,v 1.216 2007/01/10 20:23:21 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -495,7 +495,7 @@ sub get_subscribed_hosts {
     my $srcf=$2;
     opendir(DIR,$1);
     while ($filename=readdir(DIR)) {
-	if ($filename=~/\Q$srcf\E\.(\w+)$/) {
+	if ($filename=~/\Q$srcf\E\.($match_lonid)$/) {
 	    my $subhost=$1;
 	    if (($subhost ne 'meta' && $subhost ne 'subscription' &&
 		 $subhost ne 'tmp') &&
@@ -509,7 +509,7 @@ sub get_subscribed_hosts {
     if ( $sh=Apache::File->new("$target.subscription") ) {
 	&Apache::lonnet::logthis("opened $target.subscription");
 	while (my $subline=<$sh>) {
-	    if ($subline =~ /(^\w+):/) { 
+	    if ($subline =~ /^($match_lonid):/) { 
                 if ($1 ne $Apache::lonnet::perlvar{'lonHostID'}) { 
                    push(@subscribed,$1);
 	        }
