@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler to show and edit custom distribution rights
 #
-# $Id: lonrights.pm,v 1.20 2006/11/09 22:48:50 raeburn Exp $
+# $Id: lonrights.pm,v 1.21 2006/12/20 22:41:08 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -89,8 +89,8 @@ sub handler {
 		          ($rulehash{'effect'} eq 'allow')) {
 		      $rulehash{'effect'}='deny';
                   }
-                  $rulehash{'domain'}=~s/\W//g;
-                  $rulehash{'course'}=~s/\W//g;
+                  $rulehash{'domain'} = &LONCAPA::clean_domain($rulehash{'domain'});
+		  $rulehash{'course'} = &LONCAPA::clean_courseid($rulehash{'course'});
                   $rulehash{'section'}=~s/\W//g;
                   unless ($rulehash{'domain'}) { 
                      $rulehash{'domain'}=$env{'user.domain'}; 
@@ -243,8 +243,9 @@ ENDSTARTTABLE
 	  $r->print('</td><td>');
 # ---- realm
           my $realm=$token->[2]->{'realm'};
-          $realm=~s/^\W//;
           my ($rdom,$rcourse,$rsec)=split(/[\/\_]/,$realm);
+	  $rdom = &LONCAPA::clean_domain($rdom);
+	  $rcourse = &LONCAPA::clean_courseid($rcourse);
 # realm domain
           if ($constructmode) {
               unless ($rdom) { $rdom=$env{'user.domain'}; }
